#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/time.h>
#include <dirent.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "siril.h"

extern GtkWidget *medstackwindow;
extern GtkWidget *main_window;
extern GtkWidget *registration_window;
extern GtkWidget *stack_window;
extern GtkWidget *about_window;
extern GtkWidget *rgb_window;
extern GdkGC *gc;
extern GtkWidget *OpenDial;
extern cominfo com;

gboolean pending_cr=0;
char sep[128]="";
char buffercomm[128];
char buffercomm[128];
char buffer[512];
char error_buffer[256];
char currentfilename[512];
gboolean loaded=0;

	
void adjuststackspin(void){
	static GtkWidget *stackspin;
	static GtkAdjustment *stackadj;

	if(stackspin==NULL){
		stackspin=lookup_widget(main_window,"stackspin");
		}

	if(stackadj==NULL){
		stackadj=gtk_spin_button_get_adjustment( GTK_SPIN_BUTTON (stackspin) );
		}
	
	stackadj->upper=(gfloat)com.seq.selnum;
	}

double timing(int fin, char *info) {
	static struct timeval timedeb, timefin;
	double delay;

	if (fin) {
		gettimeofday(&timefin,NULL);
		delay=timefin.tv_sec + (double) timefin.tv_usec*1e-6 -
					    (timedeb.tv_sec +(double) timedeb.tv_usec*1e-6);
		sprintf(buffer,"%s : %lf\n", info, delay);
//		fprintf(stderr,"%s", buffer);
		return delay;
		}
	else{
		gettimeofday(&timedeb,NULL);
		return (double) 0;
		}
	}

void report_fits_error( int status, char *buffer) {
    if (status) {
	      	while(fits_read_errmsg(buffer)){
			siril_error(buffer,1);
			}
		siril_error(NULL,0);
		}
    return ;
}

int siril_error(char *msg, int append){
        GtkWidget *text;
        GdkFont *fixed_font;
	static int last_append;
	
        text = lookup_widget (main_window, "output");
        
        fixed_font = gdk_font_load ("-*-fixed-*-*-*-*-*-120-*-*-*-*-*-*");
        if (!text){
	         return 0;
		}
        /* Freeze the text widget, ready for multiple updates */
	gtk_text_freeze (GTK_TEXT (text)); 
	if(msg!=NULL){
		if (!last_append){
			gtk_text_set_point( GTK_TEXT(text),0);
			gtk_text_insert (GTK_TEXT (text), NULL, NULL,NULL,msg, strlen(msg));
			}
		else{
			gtk_text_insert (GTK_TEXT (text), NULL, NULL,NULL,"\n", 1);
			gtk_text_insert (GTK_TEXT (text), NULL, NULL,NULL,msg, strlen(msg));
			}
		}

	if (!append){
		if (msg==NULL){
			gtk_text_insert (GTK_TEXT (text), NULL, NULL,NULL,"\n", 1);
			}
		gtk_text_insert (GTK_TEXT (text), NULL, NULL,NULL,sep, strlen(sep));
		}
	gtk_text_thaw (GTK_TEXT (text)); 
	last_append=append;	
	return 0;
	}

gboolean inimage(GdkEvent *event){
	if (	((GdkEventButton*)event)->x > 0 && 
		((GdkEventButton*)event)->x < gfit.rx &&
		((GdkEventButton*)event)->y > 0 && 
		((GdkEventButton*)event)->y < gfit.ry){
		return TRUE;
		}
	else {
		return FALSE;
		}
	}

/* Set up the RGB buffer. */

void set_ranges(void){
	GtkWidget *widget;

	widget=lookup_widget(main_window,"scalemin");
	gtk_adjustment_set_value (GTK_RANGE (widget)->adjustment,(gfloat)gfit.lo);
	widget=lookup_widget(main_window,"scalemax");
	gtk_adjustment_set_value (GTK_RANGE (widget)->adjustment,(gfloat)gfit.hi);
	}

int adjust_exclude(int n){
	GdkPoint points[2];
	GtkWidget *widget, *draw;

	widget=lookup_widget(main_window, "exclude_button");

	 if (!com.seq.imgparam){
		 return 1;
		 }

	gtk_signal_handler_block_by_func(GTK_OBJECT( widget), on_excludebutton_toggled, NULL);
	if (com.seq.imgparam[n].incl){
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON (widget), TRUE );
			draw=lookup_widget(main_window,"drawingarea1");
			if(com.cvport==GLAYER)
					draw=lookup_widget(main_window,"drawingareag");
			if(com.cvport==BLAYER)
					draw=lookup_widget(main_window,"drawingareab");
			gdk_gc_set_function(draw->style->white_gc, GDK_XOR);
			points[0].x=0;
			points[0].y=0;
			points[1].x=gfit.rx;
			points[1].y=gfit.ry;
			gdk_draw_lines (draw->window, draw->style->white_gc, points, 2);
			points[0].y=gfit.ry;
			points[1].y=0;
			gdk_draw_lines (draw->window, draw->style->white_gc, points, 2);
			gdk_gc_set_function(draw->style->white_gc, GDK_COPY);
			}
	 else {
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON (widget), FALSE );
	 }
	gtk_signal_handler_unblock_by_func(GTK_OBJECT( widget), on_excludebutton_toggled, NULL);
	adjust_sellabel(1);

	return 0;
	}

int adjust_sellabel(int i ){
	if (i){
		snprintf(buffer, 512, 
			"<%s.seq> : %d images selected out of %d",
			com.seq.name,
			com.seq.selnum,
			com.seq.number);
		}
	else {
		sprintf(buffer, "No sequence");
		}

	gtk_label_set_text (GTK_LABEL(lookup_widget(main_window,"imagesel_label")),
				   buffer);
	return 0;
	}

int readseqfile(char *name){
	char line[256];
	char filename[256];
	int i;
	FILE *seqfile;
	int alloced=0; 

	//fprintf(stderr,"readseqfile:  sequence :%s:\n", name);

	isempty(name);
	if(!strstr(name,".seq")){
		snprintf(filename,255,"%s.seq", name);
		}
	else{
		strncpy(filename,name,255);
		}

	seqfile=fopen(filename, "r");

	com.seq.selnum=0;
	if(seqfile!=NULL){
		i=0;
		fgets(line,255,seqfile);
		do{
			if(*line!='#'){
				if(*line=='S'){
					if (line[2]=='"'){
						fprintf(stderr,"readseqfile: void sequence\n");
						if(sscanf(line+4,"%d %d %d %d", &com.seq.beg,&com.seq.number,&com.seq.selnum,
												&com.seq.fixed)!=4 || alloced!=0){
									fprintf(stderr,"readseqfile:sequence file format error: %s\n",line);
									return 1;

								}
						else{
							com.seq.name[0]=0;
								}
							}
					else{
						if(sscanf(line+2,"%s %d %d %d %d",com.seq.name,
								&com.seq.beg,&com.seq.number,&com.seq.selnum,&com.seq.fixed)!=5 || alloced!=0){
							fprintf(stderr,"readseqfile:sequence file format error: %s\n",line);
							return 1;
							}
						}
					// alloc space to store exclude information
					com.seq.imgparam=realloc(com.seq.imgparam,(com.seq.number+1)*sizeof(imgdata));
					alloced=1;
					}
				else {

			   if (sscanf(line,"%d %d %d %d %d %d", &(com.seq.imgparam[i].filenum),
									   			    &(com.seq.imgparam[i].qual_rank),
												    &(com.seq.imgparam[i].shiftx),
												    &(com.seq.imgparam[i].shifty),
												    &(com.seq.imgparam[i].quality),
												    &(com.seq.imgparam[i].incl))!=6){
					   fprintf(stderr,"Error while reading seq file %s\n", filename);
			   			}	
#if 0
				   fprintf(stderr,"%d %d %d %d %d %d \n", 
									   com.seq.imgparam[i].filenum, 
									   com.seq.imgparam[i].qual_rank, 
									   com.seq.imgparam[i].shiftx, 
									   com.seq.imgparam[i].shifty, 
									   com.seq.imgparam[i].filenum, 
									   com.seq.imgparam[i].incl);
#endif
					   ++i;
					}
				}
			}
		while(!feof(seqfile) && (fgets(line, 255, seqfile)!=NULL));

		fclose(seqfile);
		}
	else {
		fprintf(stderr,"readseqfile: could not open %s\n",filename);
		}
	com.seq.end=com.seq.beg+com.seq.number-1;
	com.seq.current=com.seq.beg;
	com.seq.status |= SEQSHOWEXC; //default is to show also excluded image	
	adjuststackspin();
	return 0;
	}

int seqsetnum(int beg, int end){
	GtkWidget *widget;
	GtkAdjustment *adj;
	
	if((widget=lookup_widget(main_window,"imagenumber_spin"))==NULL){
		snprintf(error_buffer,255,"Cannot find seqproc window\n");
		siril_error(error_buffer,0);
		return 1;
		}
		
	adj=gtk_spin_button_get_adjustment( GTK_SPIN_BUTTON (widget) );
	gtk_adjustment_set_value (adj,(gfloat)beg);

	adjust_exclude (beg);
	
	adj->lower=(gfloat)beg;
	adj->upper=(gfloat)end;
	return 0;
	}

int check_seq(char *name){ // name is a prefix, possibly NULL
	// in the current directory, 
	// if name is null, looks for sequences of fits files
	// 		and builds the corresponding sequence files
	// if name is not null, builds the corresponding 
	// 		sequence and file.
	// called when changing wd with name==NULL
	// 	or when an explicit root name is given in the GUI
	char filename[256];
	int count=0;
	DIR *dir;
	struct dirent *file;
	//
	// returns if sequence processing is not active
	//
	if(!com.seq.active){
		return TRUE;
		}
	//
	// read files in cwd and look for a sequence
	//
	// fprintf(stderr,"check_seq\n");

	if(name==NULL){
		if((dir=opendir(com.wd))==NULL){
			return TRUE;
			}

		while ((file=readdir(dir))!=NULL && count<10){
			strcpy(filename,file->d_name);
			if(getindex(filename)!=-1){ // getindex truncates filename to the presumed
									// root name
				if(!buildseqfile(filename)){
					++count;
					}
				}
			}
		return TRUE;
		}
	else{
		return buildseqfile(name);
		}
	}

int writeseqfile(char *name){// filename is without suffix .seq
	// 	writes seqfile rootname.seq
	char filename[MAXGENLENGTH];
	FILE *seqfile;
	int i;

	isempty(name);
	snprintf(filename,MAXGENLENGTH,"%s.seq", name);
	seqfile=fopen(filename,"w+");
	if(seqfile==NULL){
		fprintf(stderr,"check_seq: cannot open %s for writing\n",filename);
		return TRUE;
		}

	fprintf(seqfile,"#Include/exclude info. \n");
	fprintf(seqfile,"#Image num.    Exclude (1)\n");
	if(strlen(name)==0){
		fprintf(stderr,"S \" %d %d %d %d\n", 
			com.seq.beg, com.seq.number, com.seq.selnum, com.seq.fixed);
		fprintf(seqfile,"S \" %d %d %d %d\n", 
			com.seq.beg, com.seq.number, com.seq.selnum, com.seq.fixed);
		}
	else{
		fprintf(stderr,"S %s %d %d %d %d\n", 
			name, com.seq.beg, com.seq.number, com.seq.selnum, com.seq.fixed);
		fprintf(seqfile,"S %s %d %d %d %d\n", 
			name, com.seq.beg, com.seq.number, com.seq.selnum, com.seq.fixed);
		}

	for(i=0;i<com.seq.number;++i){
		fprintf(seqfile,"%d %d %d %d %d %d\n",  com.seq.imgparam[i].filenum, 
									com.seq.imgparam[i].qual_rank, 
									com.seq.imgparam[i].shiftx, 
									com.seq.imgparam[i].shifty, 
									com.seq.imgparam[i].quality, 
									com.seq.imgparam[i].incl); //default is to include all images, hence the zero
		fprintf(stderr,"%d %d %d %d %d %d\n",  com.seq.imgparam[i].filenum, 
									com.seq.imgparam[i].qual_rank, 
									com.seq.imgparam[i].shiftx, 
									com.seq.imgparam[i].shifty, 
									com.seq.imgparam[i].quality, 
									com.seq.imgparam[i].incl); //default is to include all images, hence the zero
		}
	fclose(seqfile);
	return FALSE;
	}

gboolean existseq(char *name){
	char filename[MAXGENLENGTH];
    struct stat sts;

	isempty(name);
	snprintf(filename,255,"%s.seq",name);
	if(stat(filename, &sts)==0){
		return TRUE;
		}
	return FALSE;
	}

int	buildseqfile(char *name) { 
		// name is a prefix, not NULL, possibly void
	int  imagetype, i;

//	if(!com.seq.active) return TRUE;
	if(existseq(name)==TRUE){
		//fprintf(stderr,"seqfile '%s' exists\n", name);
		return FALSE;
		}
	
	// check if the sequence begins at 0 or at 1
	if (stat_file(buildfilename(name,0),&imagetype) || imagetype!=TYPEFITS){
		if (stat_file(buildfilename(name,1),&imagetype) || imagetype!=TYPEFITS){
			/*com.seq.current=0;
			com.seq.selnum=0;
			com.seq.number=0;
			com.seq.beg=0;
			com.seq.end=0;*/
			sprintf(error_buffer,"No sequence %s fixed %d...\n", name, com.seq.fixed);
			siril_error(error_buffer,0);
			/*adjuststackspin();*/
			return TRUE;
			}
		else {
			com.seq.beg=1;
			}
		}
	else {
		com.seq.beg=0;
		}

	com.seq.end=com.seq.beg;
	// search how many images we have in the sequence
	while(!stat_file(buildfilename(name,com.seq.end),&imagetype) && imagetype==TYPEFITS){
		++com.seq.end;
		}
	--com.seq.end;
	com.seq.number = com.seq.end - com.seq.beg + 1;
	com.seq.imgparam=realloc(com.seq.imgparam,(com.seq.number+1)*sizeof(imgdata));
	strncpy(com.seq.name, name, 255);
	for(i=0;i<com.seq.number;++i){
		com.seq.imgparam[i].filenum=i+1;
		com.seq.imgparam[i].qual_rank=i+1;
		com.seq.imgparam[i].shiftx=0;
		com.seq.imgparam[i].shifty=0;
		com.seq.imgparam[i].quality=0;
		com.seq.imgparam[i].incl=0;
		}
	com.seq.selnum=com.seq.number;
	writeseqfile(com.seq.name);

	sprintf(error_buffer,"%s %d->%d\n", name, com.seq.beg, com.seq.end);
	siril_error(error_buffer,0);
	adjuststackspin();
	return FALSE;
	}

int set_seq(char *name){
	GtkWidget *widget;
	static GtkAdjustment *adj;

	//fprintf(stderr,"set_seq <%s>\n", name);
	if(!com.seq.active){ 
		return TRUE;
		}

	readseqfile(name);
// set limits for spin button
//	fprintf(stderr,"set_seq:seqsetnum %d %d\n",com.seq.beg,com.seq.end);
	seqsetnum(com.seq.beg, com.seq.end);
	adjust_exclude(com.seq.beg);

//	fprintf(stderr,"set_seq:readfits %s %d\n",name, com.seq.beg);
	com.seq.current=com.seq.beg;
	readfits(buildfilename(com.seq.name,com.seq.beg), &gfit);
	if(gfit.naxes[2]==3){
		rgbprocess(TRUE);
		}
	else{
		rgbprocess(FALSE);
		}
	redraw(com.cvport,REMAP);
	display_filename(com.formname);
	adjust_exclude(com.seq.beg);

	widget=lookup_widget(main_window,"imagenumber_spin");
	if(adj==NULL){
		adj=gtk_spin_button_get_adjustment( GTK_SPIN_BUTTON (widget) );
		}
	gtk_adjustment_set_value(adj, (gfloat)com.seq.beg); 

	// update filename display
	widget=lookup_widget(main_window, "filename_entry");
	gtk_entry_set_text(GTK_ENTRY(widget), com.formname);

	// initializes SEQSHOWEXC button
	widget=lookup_widget(main_window, "showexcluded_button");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widget),TRUE);	
	return 0;
}


void remaprvb(void){
	guchar *pos;
	guchar *bufr, *bufg, *bufb;
	gint i;
	int nbdata;
			
//	fprintf(stderr,"remaprvb\n");
	if(gfit.naxis==3){
		if(com.rgbbuf==NULL){
			com.rgbbuf=realloc(com.rgbbuf, (gfit.naxis*gfit.rx*gfit.ry*sizeof(WORD))<<com.zoom);
			if (com.rgbbuf==NULL){
				fprintf(stderr,"remap: error realloc\n");
				return;
				}
			}
// WARNING : this assumes that R, G and B buffers are already allocated and mapped
// it seems ok, but one can probably imagine situations where it segfaults
		bufr=com.graybuf[RED_VPORT];
		bufg=com.graybuf[GREEN_VPORT];
		bufb=com.graybuf[BLUE_VPORT];
		if(bufr==NULL||bufg==NULL||bufb==NULL){
			fprintf(stderr,"remaprvb: gray buffers not allocated for display\n");
			return;
			}
		pos = com.rgbbuf;
		nbdata=gfit.rx*gfit.ry;

		for (i = 0; i < nbdata; i++) {
			*pos++ = *bufr++;
			*pos++ = *bufg++;
			*pos++ = *bufb++;
			}
		}
	}

void remap(int vport){
	// This function maps fit data with a linear LUT between lo and hi levels
	// to the buffer to be displayed; display only is modified
	guint x, y, tmp;
	gdouble pente;
	guchar *pos;
	fits *fit=&(gfit);
	WORD *buf=fit->pdata[vport];

//	fprintf(stderr,"remap %d\n", vport);
	if (vport==RGB_VPORT){
		remaprvb();
		}
	else {
		if(buf==NULL){
			return;
			}

		if(com.graybuf[vport]==NULL){
			com.graybuf[vport]=realloc(com.graybuf[vport], (gfit.rx*gfit.ry*sizeof(WORD))<<com.zoom);
			if (com.graybuf[vport]==NULL){
				fprintf(stderr,"remap: error realloc\n");
				return;
				}
			}

		pos = com.graybuf[vport]+(fit->ry-1)*fit->rx;
		if(fit->hi==fit->lo)
			pente=1.;
		else
			pente=(double)255/(double)(fit->hi-fit->lo);
		for (y = 0; y < fit->ry; y++) {
			for (x = 0; x < fit->rx; x++) {
				tmp = max(0,((*buf)-(short)fit->lo))*pente;   
				*pos = min(tmp,255);
				pos++;
				buf++;
				}
			pos-=2*fit->rx;
			}
		}
	}

gboolean redraw(int vport, int doremap){
	GtkWidget *widget=com.vport[vport];
	static int toldx[MAXVPORT], toldy[MAXVPORT];
	int oldx, oldy;
	fits *fit=&(gfit);

	if(vport>MAXVPORT){
		fprintf(stderr,"redraw: vport=%d is larger than MAXVPORT %d\n",vport, MAXVPORT);
		}
	
	oldx=toldx[vport];	
	oldy=toldy[vport];	
		
	if(fit->rx!=oldx || fit->ry!=oldy){
		gdk_window_clear_area (widget->window, 1,1,oldx,oldy);
		}
	switch(vport){
		case RED_VPORT:
		case BLUE_VPORT:
		case GREEN_VPORT:
			if (!com.graybuf[vport])
				break;

			if (doremap==REMAP){
				if(gfit.naxis==3){
					remap(RED_VPORT);
					remap(GREEN_VPORT);
					remap(BLUE_VPORT);
					}
				else{
					remap(vport);
					}
				}
			gdk_draw_gray_image (widget->window, 
				widget->style->fg_gc[GTK_STATE_NORMAL],
				0, 0, fit->rx,fit->ry, GDK_RGB_DITHER_NONE, 
				com.graybuf[vport], fit->rx);
			if(gfit.naxis!=3)
				break;
				
		case RGB_VPORT:
			if(gfit.naxis==3){
				if(doremap==REMAP){
					remap(RGB_VPORT);
					}
				widget=com.vport[RGB_VPORT];
					if(fit->rx!=oldx || fit->ry!=oldy){
						gdk_window_clear_area (widget->window, 1,1,oldx,oldy);
						}
					gdk_draw_rgb_image (widget->window, 
					widget->style->fg_gc[GTK_STATE_NORMAL],
					0, 0, fit->rx,fit->ry, GDK_RGB_DITHER_NONE, 
					com.rgbbuf, fit->rx*3);
				}
			break;
		default:
			fprintf(stderr,"redraw: unknown viewport number %d\n",vport);
			break;
		}

	com.drawn=0;
	toldx[vport]=fit->rx;
	toldy[vport]=fit->ry;
	return FALSE;
	}

gboolean
on_minscale_changed      (GtkAdjustment *adj, gpointer         user_data)
					
{
  GtkWidget *widget; 
  char buffer[256];

  gfit.lo=(int)adj->value;
  snprintf(buffer,6,"%d", gfit.lo);
  //fprintf(stderr,"redraw from on_minscale_changed %s",buffer);
  widget=lookup_widget(main_window, "min_entry");
  gtk_entry_set_text(GTK_ENTRY(widget), buffer);
  redraw(com.cvport,REMAP);  
  return FALSE;
}


gboolean
on_maxscale_changed      (GtkAdjustment *adj,
                                        gpointer         user_data)
{
  GtkWidget *widget; 
  char buffer[256];
  
  gfit.hi=(int)adj->value;
  snprintf(buffer,6,"%d", gfit.hi);
  //fprintf(stderr,"on_maxscale_changed %s",buffer);
  //fprintf(stderr,"redraw from on_maxscale_changed %s",buffer);
  widget=lookup_widget(main_window, "max_entry");
  gtk_entry_set_text(GTK_ENTRY(widget), buffer);
  
    redraw(com.cvport,REMAP);  

  return FALSE;
}

gboolean
on_drawingarea1_expose_event           (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
//fprintf(stderr,"redrawing from on_drawingarea1_expose_event\n");
  switch (*(char *)user_data){
	case 'r':
		  redraw(RED_VPORT,NOREMAP);
		break;
	case 'g':
		  redraw(GREEN_VPORT,NOREMAP);
		break;
	case 'b':
		  redraw(BLUE_VPORT,NOREMAP);
		break;

	case 'c':// 'c' stands for 'color'
		  if(com.rgb){
			if(com.graybuf[RLAYER]==NULL)
				redraw(RED_VPORT,NOREMAP);
			if(com.graybuf[GREEN_VPORT]==NULL)
				redraw(GREEN_VPORT,NOREMAP);
			if(com.graybuf[BLUE_VPORT]==NULL)
				redraw(BLUE_VPORT,NOREMAP);

			redraw(RGB_VPORT,NOREMAP);
		  	}
		break;
	  }


  return FALSE;
}

static int whichdial;

	void opendial(void){
	if(!GTK_IS_WIDGET(OpenDial))
		OpenDial=create_OpenDial();
	gtk_widget_show (OpenDial);
	}


void
on_darkfile_button_released            (GtkButton       *button,
                                        gpointer         user_data)
{
whichdial=OD_DARK;
opendial();
}

void
on_cwd_btton_released                  (GtkButton       *button,
                                        gpointer         user_data)
{
whichdial=OD_CWD;
opendial();
}


void
on_offsetfile_button_released          (GtkButton       *button,
                                        gpointer         user_data)
{
whichdial=OD_OFFSET;
opendial();
}


void
on_flatfile_button_released            (GtkButton       *button,
                                        gpointer         user_data)
{
whichdial=OD_FLAT;
opendial();
}


void
on_open1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
whichdial=OD_OPEN;
opendial();
}

int checkdir(char *name){
	GtkWidget *seqcombo, *widget;
	GList *seqcombo_items = NULL;
	DIR *dir;
	struct dirent *file;
	char buffer[256];
	char *suf;
	char filename[16][256];
	int i;

	snprintf(buffer,255,"%s v%s - %s",PACKAGE,VERSION,name);
	gtk_window_set_title (GTK_WINDOW (main_window), buffer );

	if((dir=opendir(name))==NULL){
		return 1;
	 	}
	i=0;

	if(com.seq.active){
		while ((file=readdir(dir))!=NULL && i <255){
			if ((suf=strstr(file->d_name,".seq")) && strlen(suf)==4){
	//			fprintf(stderr,"found %s %d\n",file->d_name,i);
				strncpy(filename[i],file->d_name,255);
				seqcombo_items = g_list_append (seqcombo_items, 
							(gpointer) filename[i]);
				++i;
				}
			}
		}

	widget=lookup_widget(main_window,"seqproc_entry");	
	seqcombo=lookup_widget(main_window,"seqcombo");	
	//gtk_signal_handler_block_by_func(GTK_OBJECT( widget), on_seqproc_entry_changed, NULL);
	
	if(seqcombo_items==NULL){
		seqcombo_items = g_list_append (seqcombo_items, (gpointer) "");
		}

	gtk_combo_set_popdown_strings (GTK_COMBO (seqcombo), seqcombo_items);
	//gtk_signal_handler_unblock_by_func(GTK_OBJECT(widget), on_seqproc_entry_changed, NULL);
	g_list_free (seqcombo_items);
	return 0;
	}

int changedir(char *dir){
	char filename [256];
	int i;

	i=strlen(dir)-1;
	while (dir[i]=='/'){
		//fprintf(stderr,"in <%s> %d\n",dir, i);
		dir[i]=0;
		i=strlen(dir)-1;
		//fprintf(stderr,"got 1 / <%s> %d\n",dir, i);
		}

	chdir(dir);
	check_seq(NULL);
	checkdir(dir);
	sprintf(filename,"%s/", com.wd);
	gtk_file_selection_set_filename(GTK_FILE_SELECTION(OpenDial),filename);
	return 0;
	}

void
on_open_ok_button1_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
char *filename;
GtkWidget *widget;

filename=gtk_file_selection_get_filename(GTK_FILE_SELECTION(OpenDial));

switch(whichdial){
	case OD_FLAT:
		strncpy(com.seq.flatname, filename,255);
		widget=lookup_widget(main_window, "flatname_entry");
		if(widget){
			gtk_entry_set_text(GTK_ENTRY(widget), com.seq.flatname);
			}
		break;

	case OD_DARK:
		strncpy(com.seq.darkname, filename,255);
		widget=lookup_widget(main_window, "darkname_entry");
		if(widget){
			gtk_entry_set_text(GTK_ENTRY(widget), com.seq.darkname);
			}
		break;
		
	case OD_OFFSET:
		strncpy(com.seq.offsetname, filename,255);
		widget=lookup_widget(main_window, "offsetname_entry");
		if(widget){
			gtk_entry_set_text(GTK_ENTRY(widget), com.seq.offsetname);
			}
		break;
		
	case OD_CWD:
		strncpy(com.wd, filename,255);
		changedir(com.wd);
		writeinitfile();
		break;
		
	case OD_OPEN:
		strncpy(currentfilename, filename,255);
		readfits(filename,&(gfit));
		loaded=1;
		widget=lookup_widget(main_window,"scalemin");
		gtk_adjustment_set_value (GTK_RANGE (widget)->adjustment,
						(gfloat)gfit.lo);
		widget=lookup_widget(main_window,"scalemax");
		gtk_adjustment_set_value (GTK_RANGE (widget)->adjustment,
						(gfloat)gfit.hi);
		redraw(com.cvport,REMAP);
		break;
	}
	
	whichdial=OD_NULL;
	gtk_widget_hide (OpenDial);
}


void
on_open_cancel_button1_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
gtk_widget_hide (OpenDial);
}




void
on_save1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (loaded){
		savefits(currentfilename, &(gfit));
		}
	else {
		}
}




void
on_exit_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
exit(0);
}


void
on_command_changed                     (GtkEditable     *editable,
                                        gpointer         user_data)
{
char *text;

	text = gtk_editable_get_chars(GTK_EDITABLE(editable), 0,
	                              gtk_text_get_length(GTK_TEXT(editable)));
//fprintf(stderr, "%s", text);
}

/*

struct GdkEventKey
 {
    GdkEventType type;
       GdkWindow *window;
          gint8 send_event;
	     guint32 time;
	        guint state;
		   guint keyval;
		      gint length;
		         gchar *string;
			  };
*/			  
gboolean
on_command_key_release_event           (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{

  return FALSE;
}


gboolean
on_command_key_press_event             (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
char *text, *comm=NULL;
gint length, pos, index;

if (event->keyval==GDK_Return){
	length=gtk_text_get_length(GTK_TEXT(widget));
	pos=gtk_editable_get_position (GTK_EDITABLE(widget));
	if(GTK_TEXT_INDEX(GTK_TEXT(widget), pos)!=0x0d && pos <length)
		pending_cr=1;
	length=gtk_text_get_length(GTK_TEXT(widget));
//	fprintf(stderr,"long : %d %d \n", length, pos);
	text = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, length);
				 
	index=0;
	while ((comm=strsep(&text ,"\n"))!=NULL) {
		if (comm) 
			index += strlen(comm)+1;

		if(pos <= index){
			break;
			}
		}
	sprintf(buffercomm,"%s\n",comm);
	snprintf(com.status,255,"Working...");
	progress(0);
	processcommand(buffercomm);
	    gtk_widget_queue_draw (GTK_WIDGET (main_window));
		snprintf(com.status,255,"Ready...");
	progress(0);

	/*loaded=1;
	widget=lookup_widget(main_window,"scalemin");
	gtk_adjustment_set_value (GTK_RANGE (widget)->adjustment,(gfloat)gfit.lo);
	widget=lookup_widget(main_window,"scalemax");
	gtk_adjustment_set_value (GTK_RANGE (widget)->adjustment,(gfloat)gfit.hi);
	redraw(&gfit);*/
	}

gtk_widget_draw_focus(lookup_widget(main_window,"drawingarea1"));

return FALSE;
}



void
on_command_changed_event               (GtkEditable     *editable,
                                        gpointer         user_data)
{
if (pending_cr){
	gtk_text_backward_delete(GTK_TEXT(editable),1);
	pending_cr=0;
	}
}


gboolean erase_last(GtkWidget *widget){
		gdk_gc_set_function(widget->style->white_gc, GDK_XOR);
		gdk_draw_rectangle (widget->window,
				widget->style->white_gc,
				FALSE,
				com.lx,com.ly,
				com.llx, com.lly);
		gdk_gc_set_function(widget->style->white_gc, GDK_COPY);
		com.drawn=0;
		return FALSE;
	}

gboolean
on_drawingarea1_button_press_event     (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
if (inimage((GdkEvent *)event)){
  if (event->button == 1){
	if (com.drawing){
		com.drawing=0;
		}
	else{
		if (com.drawn)
			erase_last(widget);
	      	com.drawing=1;
		com.upx=event->x;
		com.upy=event->y;
		}
	}	
  
  if (event->button == 2){
  	com.leveldrag=1;
	if(com.autolevel){
		com.autolevel=FALSE;
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(main_window,"auto_level_button1")), FALSE);
		}
  	}
  }
  return FALSE;
}

gboolean
on_drawingarea1_button_release_event   (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
if (inimage((GdkEvent *)event)){
  if (event->button == 1 ){
	if (com.drawing){
	      	com.drawing=0;
		com.dnx=event->x;
		com.dny=event->y;
		}
	}	

  if (event->button == 2){
  	com.leveldrag=0;
  	}
  }
  return FALSE;
}


gboolean
on_drawingarea1_motion_notify_event    (GtkWidget       *widget,
                                        GdkEventMotion  *event,
                                        gpointer         user_data)
{
  gint dum;
  static int delay;
  gchar label[32]="labeldensity";
  fits *fit=&(gfit);
  
  delay=min(5,delay);
  if (	inimage((GdkEvent *)event) ){
	  if(com.leveldrag){
		  fit->lo=(event->y*(double)fit->max[com.cvport]/2/(double)fit->ry);
		  fit->hi=(event->x*(double)fit->max[com.cvport]*2/(double)fit->rx);
		  --delay;
		  if(delay<=0){
			  level_adjust(fit);  
			  delay=5;
			  }
	  	}
	  else {
		sprintf (buffer, "x: %.3d y: %.3d = %d", 
			(gint)event->x, 
			fit->ry-1-(gint)event->y,
			fit->data[(gint)(fit->rx*(gint)(fit->ry - event->y -1 ) 
				+(gint)event->x)]);

		strncat(label,user_data,32);
		gtk_label_set_text (GTK_LABEL(lookup_widget(main_window,label)), buffer);

		if (com.drawing){
			if (com.drawn)
				erase_last(widget);
			com.dnx=event->x;
			com.dny=event->y;
			if (com.upx>com.dnx){
				dum=com.dnx; com.dnx=com.upx; com.upx=dum;
				}
			if (com.upy>com.dny){
				dum=com.dny; com.dny=com.upy; com.upy=dum;
				}

			com.lx=com.upx; com.ly=com.upy; 
		  	com.cly=fit->ry-com.ly;
			com.llx=com.dnx-com.upx; com.lly=com.dny-com.upy;
			gdk_gc_set_function(widget->style->white_gc, GDK_XOR);
			gdk_draw_rectangle (widget->window,
				widget->style->white_gc,
				FALSE,
				com.lx,com.ly,
				com.llx, com.lly);
			gdk_gc_set_function(widget->style->white_gc, GDK_COPY);
			com.drawn=1;
			}
		}
	}
	return FALSE;
}

void
on_registration1_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if(!com.drawn){
		snprintf(error_buffer,255,"Select a region first\n");
		siril_error(error_buffer,0);
		}
	else {
		gtk_widget_show(registration_window);
		}
}


void
on_reg_cancel_button_released          (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide(registration_window);
}


void
on_reg_ok_button_released              (GtkButton       *button,
                                        gpointer         user_data)
{
	char *filename;
	int cx, cy, debx, deby, size;
	int number;

	gtk_widget_hide(registration_window);
	
	if (com.drawn){
		debx=com.lx;
		deby=com.cly-com.lly;
		size=max(com.llx,com.lly);
		cx=debx+com.llx/2;
		cy=deby+com.lly/2;
		}
	else {
		return;
		}

	filename=gtk_entry_get_text(GTK_ENTRY(lookup_widget(registration_window,"reg_filename_entry")));
	number=atoi(gtk_entry_get_text(GTK_ENTRY(lookup_widget(registration_window,"reg_nbimage_entry"))));
	if (number<2 || number >100000){
		snprintf(error_buffer,255,"That number (%d) does not look reasonable\n",number);
		}
	//fprintf (stderr,"registering %s %d %d %d %d\n",filename,number,cx,cy,size);
	_register(filename,number,cx,cy,size,com.reglayer); 
	
}


void
on_stack1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gtk_widget_show(stack_window);
}


void
on_stack_ok_button_released            (GtkButton       *button,
                                        gpointer         user_data)
{
	char *filename;
	char *shiftfilename;
	int number,j;

	gtk_widget_hide(stack_window);
	filename=gtk_entry_get_text(GTK_ENTRY(lookup_widget(stack_window,"stack_filename_entry")));
	number=atoi(gtk_entry_get_text(GTK_ENTRY(lookup_widget(stack_window,"stack_nbimage_entry"))));

	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(user_data))==TRUE){
		shiftfilename=gtk_entry_get_text(GTK_ENTRY(lookup_widget(stack_window,"stack_shiftfile_entry")));
		j=snprintf(error_buffer,255,"Could not find shift file %s", shiftfilename);
		if (stat_file(shiftfilename, &(com.imagetype))){
			snprintf(shiftfilename,255,"%s.shift", filename);
			if (stat_file(shiftfilename, &(com.imagetype))){
				snprintf(error_buffer+j,255-j,", %s", shiftfilename);
				siril_error(error_buffer,0);
				return ;
				}
			}
		snprintf(error_buffer,255,"Stacking, using %s as shift file\n",shiftfilename);
		siril_error(error_buffer,0);
		composit(filename,number,1);
		}
	else {
		composit(filename,number,1); // eventually FIXME
		}
	set_ranges();
}
#if 0
	if (com.seq.fixed_length_index)
		sprintf(format,	"%%s%%.%dd", com.seq.fixed_length_index
#endif
void
on_stack_cancel_button_released        (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide(stack_window);
}


void
on_median_stack1_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gtk_widget_show(medstackwindow);

}


void
on_auto_level_button1_toggled          (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (com.autolevel){
		com.autolevel=FALSE;
		}
	else {
		com.autolevel=TRUE;
		level_adjust(&gfit);
		}
}


void
on_medstack_ok_btton_released          (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_medstack_cancel_button_released     (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide(medstackwindow);

}



void
on_max_entry_changed                   (GtkEditable     *editable,
                                        gpointer         user_data)
{
  char *buf;

  buf=gtk_entry_get_text(GTK_ENTRY(editable));

  gfit.hi=atoi(buf);
  set_ranges();
}


void
on_min_entry_changed                   (GtkEditable     *editable,
                                        gpointer         user_data)
{
  char *buf;

  buf=gtk_entry_get_text(GTK_ENTRY(editable));

  gfit.lo=atoi(buf);
  set_ranges();
}


void
on_index_length_spinbutton_changed     (GtkEditable     *editable,
                                        gpointer         user_data)
{
	com.seq.fixed=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(editable));
	check_seq(NULL);
}

gboolean
on_main_window_key_press_event         (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
  if (event->keyval==GDK_Escape && (com.busy & ST_BUSY)){
	com.busy|=ST_CANCEL;  
  	}
  return FALSE;
}


void
on_about_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gtk_widget_show(about_window);
}


void
on_button1_released                    (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide(about_window);
}

void
on_usedarkbutton_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{

}


void
on_useoffsetbutton_toggled             (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{

}


void
on_useflatbutton_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{

}


void
on_excludebutton_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active (togglebutton)==TRUE){
		com.seq.imgparam[com.seq.current].incl=1;
		--com.seq.selnum;
		}
	else{
		com.seq.imgparam[com.seq.current].incl=0;
		++com.seq.selnum;
		//redraw(com.cvport);
	}
	adjuststackspin();
	adjust_exclude(com.seq.current);
	writeseqfile(com.seq.name);
}

void display_filename  (char *name){
	GtkWidget *widget;
	
	widget=lookup_widget(main_window, "filename_entry");
	if(widget){
		gtk_entry_set_text(GTK_ENTRY(widget), name);
		}
	}

void
on_imagenumberspin_changed             (GtkEditable     *editable,
                                        gpointer         user_data)
{
	GtkAdjustment *adj;
	int index;

	adj=gtk_spin_button_get_adjustment( GTK_SPIN_BUTTON (editable) );
	if(adj){
		index=adj->value;
		com.seq.current=index;
		if(index>com.seq.number || !(com.seq.imgparam) ||index>com.seq.end||index<com.seq.beg){
			return;
			}

		if ((com.seq.status & SEQSHOWEXC) || !com.seq.imgparam[index].incl){
			// excluded images shall not be displayed
			buildfilename(com.seq.name,index);
			readfits(com.formname,&(gfit));
			display_filename  (com.formname);
			//fprintf(stderr,"redrawing %s on port %d from imagenumspinchanged\n",com.formname,com.cvport);
			
			redraw(com.cvport,REMAP);
			adjust_exclude(index);
			}
		}
	}


gboolean
on_imagenumberspin_key_release_event   (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
	GtkWidget *owidget;
	GtkAdjustment *adj;
	int n;

	adj=gtk_spin_button_get_adjustment( GTK_SPIN_BUTTON (widget) );
	n=(int)adj->value;
	com.seq.current=n;

	if(n>com.seq.number || !com.seq.imgparam){
		return 1;
		}
	owidget=lookup_widget(main_window, "exclude_button");

	 if (event->keyval==GDK_Return){
		 if (com.seq.imgparam[n].incl){
			com.seq.imgparam[n].incl=0;
			++com.seq.selnum;
		 	}
		 else {
			com.seq.imgparam[n].incl=1;
			--com.seq.selnum;
			}
		 snprintf(error_buffer,255,", toggling this one\n");
		 siril_error(error_buffer,0);
		adjuststackspin();
		 adjust_exclude(n);
		 writeseqfile(com.seq.name);

/*		 if(n<com.seq.number){
				 gtk_adjustment_set_value(adj, (gfloat)(n+1)); 
				 com.seq.current=n+1;
				 adjust_sellabel(1);
		 	}
		 else{
		 	adjust_exclude(n);
		 	}
*/
		 }
  return FALSE;
}


void
on_seqexcludeall_button_released       (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	
	widget=lookup_widget(confirm_dialog,"confirmlabel");
	com.confirm=CD_EXCALL;
	gtk_label_set_text (GTK_LABEL(widget), 
		" Exclude all images ? \n (this erases previous image selection \n ... and there's no undo)");
	gtk_widget_show (confirm_dialog);
}

void
on_seqselectall_button_released        (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	
	widget=lookup_widget(confirm_dialog,"confirmlabel");
	com.confirm=CD_INCALL;
	gtk_label_set_text (GTK_LABEL(widget), 
		" Include all images ? \n (this erases previous image selection \n ... and there's no undo)");
	gtk_widget_show (confirm_dialog);
	}

void seqselectall_confirmed (void ) {
	GtkWidget *widget;
	GtkAdjustment *adj;
	int i;
	
	widget=lookup_widget(main_window,"imagenumber_spin");
	adj=gtk_spin_button_get_adjustment( GTK_SPIN_BUTTON (widget) );
	for(i=0;i<=com.seq.number;++i){
		com.seq.imgparam[i].incl=0;
		}
	adjust_exclude((int)adj->value);
	com.seq.selnum=com.seq.number;
	adjust_sellabel(1);
	adjuststackspin();
	writeseqfile(com.seq.name);
}

void seqexcludeall_confirmed (void) {
	GtkWidget *widget;
	GtkAdjustment *adj;
	int i;

	widget=lookup_widget(main_window,"imagenumber_spin");
	adj=gtk_spin_button_get_adjustment( GTK_SPIN_BUTTON (widget) );
	for(i=0;i<=com.seq.number;++i){
		com.seq.imgparam[i].incl=1;
		}
	adjust_exclude((int)adj->value);
	com.seq.selnum=0;
	adjust_sellabel(1);
	adjuststackspin();
	writeseqfile(com.seq.name);
}

void
on_writeschedule_button_released       (GtkButton       *button,
                                        gpointer         user_data)
{
	writeseqfile(com.seq.name);
	adjust_sellabel(1);
	checkdir(com.wd);
	}

void
on_prepro_button_released              (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *widget;

	// if dark selected
	widget=lookup_widget(main_window, "usedark_button");
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(widget))==TRUE){
		com.preprostatus|=USE_DARK;
		}
	// if flat selected
	widget=lookup_widget(main_window, "useflat_button");
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(widget))==TRUE){
		com.preprostatus|=USE_FLAT;
		}
	// if offset selected
	widget=lookup_widget(main_window, "useoffset_button");
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(widget))==TRUE){
		com.preprostatus|=USE_OFFSET;
		}

	seqpreprocess();
}


void
on_seqregister_button_released         (GtkButton       *button,
                                        gpointer         user_data)
{
	char filename[256];
	int cx, cy, debx, deby, size;

	if (com.drawn){
		debx=com.lx;
		deby=com.cly-com.lly;
		size=max(com.llx,com.lly);
		cx=debx+com.llx/2;
		cy=deby+com.lly/2;
		}
	else {
		siril_error("Select a rectangle first\n",0);
		return ;
		}	
	buildseqname(com.seq.selprefix, filename,-1);
	readseqfile(filename);
	_register(filename,com.seq.selnum,cx,cy,size,com.reglayer); 
	writeseqfile(com.seq.name);
}


void
on_seqstack_button_released            (GtkButton       *button,
                                        gpointer         user_data)
{
	//char shift[256];
	//int type;
	static GtkWidget *widget;
	static GtkAdjustment *adj;

	if(widget==NULL){
		widget=lookup_widget(main_window,"stackspin");
		}

	if(adj==NULL){
		adj=gtk_spin_button_get_adjustment( GTK_SPIN_BUTTON (widget) );
		}

	//buildseqname(com.seq.selprefix, filename,-1);
	//	snprintf(shift,255,"%s.seq",com.seq.name);
	//if (!stat_file(shift,&type)){
		composit(com.seq.name,adj->value,1); 
	//	}
	//else {
	//	composit(com.seq.name,adj->value,0); 
	//	}
	redraw(com.cvport,REMAP);
}

gboolean
on_mainseqname_entry_changed           (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
	char *name;

	if (event->keyval==GDK_Return){
		name=gtk_entry_get_text(GTK_ENTRY(widget));
//		strncpy(com.seq.name,name,MAXGENLENGTH);
		if(!check_seq(name)){ // builds the sequence and sequence file
			//fprintf(stderr,"set_seq: from mainseqname :%s:\n", name);
			set_seq(name);	 // assigns it to com.seq 
			return FALSE;
			}
		}
	return TRUE;
}

void
on_showexcluded_button_toggled         (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	
	widget=lookup_widget(main_window, "showexcluded_button");
	if((gtk_toggle_button_get_active (togglebutton)==FALSE)){
		com.seq.status &= ~SEQSHOWEXC; //default is to show also excluded image
		}
	else{
		com.seq.status |= SEQSHOWEXC; //default is to show also excluded image
		}		

}

void
on_seqmedianstack_button_released      (GtkButton       *button,
                                        gpointer         user_data)
{
	char filename [256];
	char name [256];
	int type;

	buildseqname(com.seq.selprefix, filename,-1);
	snprintf(name,255,"%s.shift",com.seq.selprefix);
	if (!stat_file(name,&type)){
		medstack(com.seq.selprefix, com.seq.selnum,".medstack"); 
		}
	else {
		medstack(com.seq.selprefix, com.seq.selnum,".medstack"); 
		}

	strncpy(name,".medstack",255);
	readfits(name,&(gfit));
	redraw(com.cvport,REMAP);
}


void
on_selprefix_entry_changed            (GtkEditable     *editable,
                                        gpointer         user_data)
{
	char *name;

	name=gtk_entry_get_text(GTK_ENTRY(editable));
	strncpy(com.seq.selprefix,name,MAXGENLENGTH);
}


void
on_darkname_entry_changed              (GtkEditable     *editable,
                                        gpointer         user_data)
{
	char *name;

	name=gtk_entry_get_text(GTK_ENTRY(editable));
	strncpy(com.seq.darkprefix,name,MAXGENLENGTH);

}


void
on_offsetname_entry_changed            (GtkEditable     *editable,
                                        gpointer         user_data)
{
	char *name;

	name=gtk_entry_get_text(GTK_ENTRY(editable));
	strncpy(com.seq.offsetprefix,name,MAXGENLENGTH);

}


void
on_flatname_entry_changed              (GtkEditable     *editable,
                                        gpointer         user_data)
{
	char *name;

	name=gtk_entry_get_text(GTK_ENTRY(editable));
	strncpy(com.seq.flatprefix,name,MAXGENLENGTH);
}


void
on_preproseqname_entry_changed         (GtkEditable     *editable,
                                        gpointer         user_data)
{
	char *name;

	name=gtk_entry_get_text(GTK_ENTRY(editable));
	strncpy(com.seq.ppprefix,name,MAXGENLENGTH);
}


void
on_selseqprefix_entry_changed          (GtkEditable     *editable,
                                        gpointer         user_data)
{

}

void
on_seqproc_entry_changed               (GtkEditable     *editable,
                                        gpointer         user_data)
{
	char *name;

	name=gtk_entry_get_text(GTK_ENTRY(editable));
	set_seq(name);
}

void
on_notebook1_switch_page               (GtkNotebook     *notebook,
                                        GtkNotebookPage *page,
                                        gint             page_num,
                                        gpointer         user_data)
{
	com.cvport=page_num;
}

void rgbprocess(gboolean value){
	com.rgb=value;
	
	if (value){
		gtk_label_set_text (GTK_LABEL(lookup_widget(main_window,"rgblabel")), "rgb ");
		gtk_widget_show (rgb_window);
		}
	else {
		gtk_label_set_text (GTK_LABEL(lookup_widget(main_window,"rgblabel")), "gray");
		gtk_widget_hide (rgb_window);
		}
	setlayers();
	}

void
on_rgb_button_released                 (GtkButton       *button,
                                        gpointer         user_data)
{
	rgbprocess(!com.rgb);
}

int	getindex(char *filename){
	// returns an index found in filename and filename is truncated to
	// the presumed root name
	char buffer[MAXGENLENGTH];
	char *suffix, *endroot;
	int i;

	strncpy(buffer,filename,MAXGENLENGTH);
	suffix=strstr(buffer,".fit");
	if(suffix==NULL){ 
		return -1;	// implies that fits files contain the extension .fit(s)
		suffix=buffer+strlen(buffer)-1;
		while(!isdigit(*suffix--) && suffix >=buffer);
		if(suffix<buffer){
			return -1;
			}
		++suffix;
		}
	*suffix--=0;

	if (suffix<buffer || (!isdigit(*suffix))){
		return -1;
		}

	while(isdigit(*--suffix) && suffix>=buffer);

	++suffix;
	i= atoi(suffix);
	endroot=strstr(filename,suffix);
	*endroot=0;
	return i;
	}


void
on_notebook2_switch_page               (GtkNotebook     *notebook,
                                        GtkNotebookPage *page,
                                        gint             page_num,
                                        gpointer         user_data)
{
	switch(page_num){
		case SELPAGE:	
			com.seq.active=TRUE;
			/*fprintf(stderr,"activate,going to check_seq\n");
			check_seq(NULL);
			checkdir(com.wd);*/
			break;
		}

}


void
on_checkseqbutton_released             (GtkButton       *button,
                                        gpointer         user_data)
{
	com.seq.active=TRUE;
	check_seq(NULL);
	checkdir(com.wd);
}


void
on_confirmok_released                  (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide (confirm_dialog);
	switch(com.confirm){
		case CD_INCALL:
			seqselectall_confirmed();
			break;	

		case CD_EXCALL:
			seqexcludeall_confirmed();
			break;	
		}
	com.confirm=CD_NULL;
}


void
on_confirmcancel_released              (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide (confirm_dialog);
	com.confirm=CD_NULL;
}

gboolean
on_drawingarea_key_press_event (GtkWidget       *widget,
                                GdkEventKey     *event,
                                gpointer         user_data)
{
static gint oldzoom;

//fprintf(stderr,"Zoom : %d\n", com.zoom);
oldzoom=com.zoom;
if (event->keyval==GDK_plus){
	com.zoom=min(ZOOMMAX,oldzoom+1);
	}

if (event->keyval==GDK_minus){
	com.zoom=max(ZOOMMIN,oldzoom-1);
	}

if (com.zoom!=oldzoom){
	redraw(com.cvport,REMAP);
	}

return FALSE;
}
