#if 0
  Siril 
  Copyright (C) 2005 Francois Meyer (dulle @ free.fr)
  Reference site is http://dulle.free.fr/alidade/siril.php

  Siril is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <stdio.h>
#include <complex.h>
#include <math.h>
#include <string.h>
#include <fftw3.h>
#include "callbacks.h"
#include "siril.h"
#include "Def_Math.h"
#include "Def_Wavelet.h"
#define EXIT 99
#define MAXCOMMAND 256
#define MAXWORD 16
#define MAXCHAR 128

extern cominfo com;
char word[MAXWORD][MAXCHAR];
int debug=0;
int status; // fitsio status
char error_buffer[256];

command commande[MAXCOMMAND]={
	{"load", 1, "load filename [rvbag]", process_load}, 
	{"save", 1, "save filename [rvbag]", process_save}, 
	{"savergb", 1, "save filename [rvbag]", process_save}, 
	{"convert", 1, "convert filename", process_convert}, 
		/* i*** commands oper filename and curent image */
	{"iadd", 1, "iadd filename", process_imoper}, 
	{"isub", 1, "isub filename", process_imoper}, 
	{"imul", 1, "imul filename", process_imoper}, 
	{"idiv", 1, "idiv filename", process_imoper},
	{"fdiv", 2, "fdiv filename scalar", process_fdiv},
		/* i***2 commands oper the genname sequence and filename*/
	{"iadd2", 4, "iadd2 genname outname filename number", process_imoper2},
	{"isub2", 4, "isub2 genname outname filename number", process_imoper2},
	{"imul2", 4, "imul2 genname outname filename number", process_imoper2}, 
	{"idiv2", 4, "idiv2 genname outname filename number", process_imoper2},
		/* s*** commands oper scalar and curent image */
	{"sadd", 1, "sadd scalar", process_soper},
	{"smul", 1, "smul scalar", process_soper}, 
	{"sdiv", 1, "sdiv scalar", process_soper},
	{"ssub", 1, "ssub scalar", process_soper},
		/* s***2 commands oper the genname sequence and scalar */
	{"sadd2", 4, "sadd2 genname outname scalar number", process_soper2},
	{"smul2", 4, "smul2 genname outname scalar number", process_soper2}, 
	{"sdiv2", 4, "sdiv2 genname outname scalar number", process_soper2},
	{"ssub2", 4, "sdiv2 genname outname scalar number", process_soper2},
		/* crop commands crop current image or genname sequence */
	{"crop", 0, "crop [x y width height]", process_crop}, 
	{"crop2", 3, "crop2 genname outname number [x y width height]", process_crop2}, 
		/* register a sequence of images */
	{"register", 2, "register genname number [size x y]", process_register},
		/* stacking  a sequence using genname.shift */
	{"composit", 2, "composit genname number shifted", process_composit},	
	{"medstack", 2, "medstack genname number outfile ", process_medstack}, 
		/* shift current image by x,y */
	{"shift", 2, "shift x y", process_shift}, 
	{"shift2", 5, "shift2 in out n x y", process_shift2}, 
		/* effectively shifts first n images sequence "in", output goes to sequence "out" */
	{"rshift2", 4, "rshift2 in out n shiftfile", process_rshift2}, 
		/* unsharp masking of current image or genname sequence */
	{"unsharp", 2, "unsharp sigma multi", process_unsharp},	
	{"unsharp2", 5, "unsharp2 sigma multi src dest number", process_unsharp2},
	{"gauss", 1, "gauss sigma ", process_gauss},	
	{"gauss2", 4, "gauss sigma src dest number", process_gauss2},	
		/* wavelet transform in nbr_plan plans */ 
	{"wavelet", 1, "wavelet nbr_plan type (1=linear 2=spline)", process_wavelet},  
		/* reconstruct from wavelet transform and weighs plans with c1, c2, c3... */ 
	{"wrecons", 2, "wrecons c1 c2 c3 ...", process_wrecons},  
		/* displays genname sequence from start_im to end_im with delay in 0.1 s and stride stride */
	{"animate", 3, "animate genname start_im end_im delay stride", process_animate},  
	
	{"bmp2fits", 0, "bmp2fits", process_bmp2fits}, /* tranforme le bmp de grvb en fits dans (wfit[0]), (wfit[1]), (wfit[2] */
	{"cd", 1, "cd directory ", process_cd},  /* subs a file to (wfit[1]) or to file2 if non zero*/
	{"composervb", 1, "composervb filename", process_composervb}, /* compose et sauve un bmp a partir de (wfit[0]), (wfit[1]), (wfit[2] */
	{"trichro", 3, "trichro rfile vfile bfile", process_trichro}, /* compose et affiche un rvb a partir de rfile, vfile, bfile */
	{"loadbmp", 1, "loadbmp filename", process_load}, /* charge un bmp dans grvb */
		/* a charge les trois fichiers filename_r filename_v filename_b */
	{"entropy", 1, "entropy filename", process_entropy},  /* computes the entropy of filename */
	{"log", 0, "log ", process_log},  /* logarifies current image */
	{"threshlo", 1, "threshlo level ", process_threshlo},  /* threshes lo (eg threshlo 0 eliminates negative values */
	{"threshhi", 1, "threshi level ", process_threshhi},  /* threshes lo (eg threshhi 32000 eliminates values above 32768*/
	{"thresh", 2, "thresh hi lo ", process_thresh},  /* threshes hi and lo */
	{"nozero", 1, "nozero level ", process_nozero},  /* replaces null values by level */
	{"ddp", 3, "ddp level coef sigma", process_ddp}, /* replaces null values by level */
	{"rrgb", 1, "file [file file]", process_rrgb}, /* 1 rgb file or 3 r,g,b files */
	{"grgb", 1, "file [file file]", process_grgb}, /* 1 rgb file or 3 r,g,b files */
	{"brgb", 1, "file [file file]", process_brgb}, /* 1 rgb file or 3 r,g,b files */
	{"lrgb", 2, "file file [file file]", process_lrgb},/* 1 lum file + (1 rgb file or 3 r,g,b files */
	{"fwhm", 0, "fwhm [x y width height]", process_fwhm}, 
	{"help", 0, "help", process_help}, /* affiche les commandes disponibles */
	{"exit", 0, "exit", process_exit},	/* exit */
	{"",0,"",0}
	};

int process_load(int nb){
	char buffer[256];
	int err=0;
	int nbdata;

	strncpy(buffer,word[1],255);

	if (strstr(buffer,".bmp")||strstr(word[0],"bmp")){
		wfit->bitpix=SIZESHORT;
		readbmp(buffer, wfit);
		}
	else{
		readfits(buffer,wfit);
		}

	nbdata=wfit->rx*wfit->ry;
	if(wfit->naxis==3){
		rgbprocess(TRUE);
//		fprintf(stderr,"RGB ok\n");
		
		copyfits(wfit,&gfit,CP_FORMAT|CP_ALLOC|CP_COPYA,0);
		gfit.pdata[RLAYER]=gfit.data;
		gfit.pdata[GLAYER]=gfit.data+nbdata;
		gfit.pdata[BLAYER]=gfit.data+nbdata*2;
		com.rgbbuf=realloc(com.rgbbuf,wfit->rx*wfit->ry*3*sizeof(WORD));
		com.graybuf[RED_VPORT]=realloc(com.graybuf[RED_VPORT],
					wfit->rx*wfit->ry*sizeof(WORD));
		com.graybuf[GREEN_VPORT]=realloc(com.graybuf[GREEN_VPORT],
						wfit->rx*wfit->ry*sizeof(WORD));
		com.graybuf[BLUE_VPORT]=realloc(com.graybuf[BLUE_VPORT],
						wfit->rx*wfit->ry*sizeof(WORD));
		}
	else{
		rgbprocess(FALSE);
		if (gfit.rx!=wfit->rx || gfit.ry!=wfit->ry || gfit.data==NULL){
			gfit.data=realloc(NULL,nbdata*3*sizeof(WORD));
			gfit.pdata[RLAYER]=gfit.data;
			gfit.pdata[GLAYER]=gfit.data+nbdata;
			gfit.pdata[BLAYER]=gfit.data+nbdata*2;
			com.rgbbuf=realloc(com.rgbbuf,wfit->rx*wfit->ry*3*sizeof(WORD));
			}
		copyfits(wfit,&gfit,CP_FORMAT,0);

		if (nb==3){
			switch(word[2][0]){
				case 'r':
					com.graybuf[RED_VPORT]=realloc(com.graybuf[RED_VPORT],
						wfit->rx*wfit->ry*sizeof(WORD));
					memcpy(gfit.pdata[RLAYER], wfit->data, nbdata*sizeof(WORD));
					break;

				case 'g':
					com.graybuf[GREEN_VPORT]=realloc(com.graybuf[GREEN_VPORT],
						wfit->rx*wfit->ry*sizeof(WORD));
					memcpy(gfit.pdata[GLAYER], wfit->data, nbdata*sizeof(WORD));
					break;

				case 'b':
					com.graybuf[BLUE_VPORT]=realloc(com.graybuf[BLUE_VPORT],
						wfit->rx*wfit->ry*sizeof(WORD));
					memcpy(gfit.pdata[BLAYER], wfit->data, nbdata*sizeof(WORD));
					break;

				default:
					com.graybuf[com.cvport]=realloc(com.graybuf[com.cvport],
						wfit->rx*wfit->ry*sizeof(WORD));
					memcpy(gfit.pdata[com.cvport], wfit->data, nbdata*sizeof(WORD));
					break;
				}
			}
		else{
			com.graybuf[com.cvport]=realloc(com.graybuf[com.cvport],
				wfit->rx*wfit->ry*sizeof(WORD));
			memcpy(gfit.pdata[com.cvport], wfit->data, nbdata*sizeof(WORD));
			}
		}
	redraw(com.cvport,REMAP);
	return err;
	}

int process_convert(int nb){
	int type;
	char suffix[16];

	if (findtype(word[1], suffix, &type)){
		snprintf(error_buffer,255,"Dont know how to convert %s.%s\n",word[1], suffix);
		siril_error(error_buffer,0);
		return 1;
		};
	snprintf(error_buffer,255,"Convert is temporarily out of service.\n Retry with next version.. \n");
	siril_error(error_buffer,0);
// FIXME	convert(word[1], suffix,type);
	return 0;
	}

int process_trichro(int nb){
	fprintf(stderr, "process_trichro is obsolete\n");
	return 0;
	}

int process_loadbmp(int nb){
	fprintf(stderr,"process_loadbmp is now process_load.\n");
	//
	return 0;
	}

int process_composervb(int nb){
	fprintf(stderr,"composervb: is obsolete\n");
	return 0;
	}

int process_bmp2fits(int nb){
	// FIXME 
	fprintf(stderr,"process_bmp2fits: temporarily out of service\n");
	return 0;
	}

int process_save(int nb){
	char buffer[256];

	sprintf(buffer,"%s", word[1]);

	if (gfit.naxis==2){
		savefits( buffer,&(gfit));	
		}

	if (gfit.naxis==3){
		memcpy(wfit,&gfit,sizeof(fits));
		wfit->naxes[2]=1;
		wfit->naxis=2;
		if (nb==3){
			switch(word[2][0]){
				case 'r':
					wfit->data=gfit.pdata[RLAYER];
					savefits( buffer, wfit );
					break;

				case 'g':
					wfit->data=gfit.pdata[GLAYER];
					savefits( buffer,wfit);
					break;

				case 'b':
					wfit->data=gfit.pdata[BLAYER];
					savefits( buffer,wfit);
					break;

				case 'a':
					sprintf(buffer,"%s_r", word[1]);
					wfit->data=gfit.pdata[RLAYER];
					savefits( buffer,wfit);
					sprintf(buffer,"%s_v", word[1]);
					wfit->data=gfit.pdata[GLAYER];
					savefits( buffer,wfit);
					sprintf(buffer,"%s_b", word[1]);
					wfit->data=gfit.pdata[BLAYER];
					savefits( buffer,wfit);
					break;
				}
			}
		else{
					savefits( buffer,&(gfit));
			}
		}
	return 0;
	}

int process_imoper(int nb){
	if(nb==2){
		readfits(word[1], &(wfit[4]));
		}

	imoper(&gfit,&wfit[4],word[0][1]);
	redraw(com.cvport,REMAP);
	return 0;
	}

// scalar add
int process_soper(int nb){ 
	float scalar;

	scalar=atof(word[1]);
	soper(&gfit,scalar,word[0][1]);
	redraw(com.cvport,REMAP);
	return 0;
	}

int process_imoper2(int nb){
	int i;
	int number;

	number=atoi(word[4]);
	readfits(word[3], &(wfit[4]));
	for (i=1;i<=number;++i){
		readfits(buildfilename(word[1],i), &(gfit));
		process_imoper(0);
		savefits(buildfilename(word[2],i),&(gfit));
		}
	return 0;
	}


int process_soper2(int nb){
	int i;
	int number;
	char in[256];

	number=atoi(word[4]);
	strncpy(in,word[1],255);
	strcpy(word[1],word[4]);
	for (i=1;i<=number;++i){
		readfits(buildfilename(in,i), &(gfit));
		process_soper(0);
		savefits(buildfilename(word[2],i),&(gfit));
		}
	return 0;
	}


int process_fdiv(int nb){
// combines an image division and a scalar multiplication.
	float norm;
	
	norm=atof(word[2]);
	readfits(word[1], &(wfit[4]));
	fdiv(&gfit,&wfit[4],norm);
	redraw(com.cvport, REMAP);
	return 0;
}

int process_composit(int nb){
	isempty(word[1]);
	if (nb==4){
		composit(word[1], atoi(word[2]), (char)atoi(word[3])); 
		}
	else {
		composit(word[1], atoi(word[2]), 1); 
		}
	//level_adjust(&com.g);
	redraw(com.cvport, REMAP);
	return 0;
	}

int process_shift(int nb){
	shift(atoi(word[1]), atoi(word[2])); 
	redraw(com.cvport, REMAP);
//	level_adjust(&com.g);
	return 0;
	}

int process_shift2(int nb){
	int i, n, x, y;

	isempty(word[1]);
	isempty(word[2]);
	n=atoi(word[3]);
	x=atoi(word[4]);
	y=atoi(word[5]);

	for(i=1;i<=atoi(word[3]);++i){
		buildfilename(word[1],i);
		readfits(com.formname,&gfit);
		shift(x,y); 
		buildfilename(word[2],i);
		savefits(com.formname,&gfit);
		}
	return 0;
	}

int process_rshift2(int nb){
	isempty(word[1]);
	isempty(word[2]);
	if (nb==5){
		rshift2(word[1], word[2], atoi(word[3]), word[4]); 
		}
	level_adjust(&gfit);
	return 0;
	}

int process_entropy(int nb){
	char buffer[256];
	double ent;

	if (stat_file(word[1],&(com.imagetype))){
		return 1;
		}
	readfits(word[1],&(wfit[4]));
	status=0;
	report_fits_error(status,buffer);
	ent=entropy(wfit[4].data,wfit[4].rx,wfit[4].ry);
	snprintf(buffer,256,"entropy %s : %10.2f\n", word[1], ent);
	//output(buffer); // to be defined in callbacks.c for output in the GUI
	return 0;
	}

int process_gauss(int nb){
//	fprintf(stderr,"%f %d\n",atof(word[1]),atof(word[2]));
	unsharp(&(gfit), atof(word[1]), (double)0);
	redraw(com.cvport, REMAP);
//	level_adjust(&(wfit[1]));
	return 0;
	}


int process_unsharp(int nb){
//	fprintf(stderr,"%f %d\n",atof(word[1]),atof(word[2]));
	unsharp(&(gfit), atof(word[1]), atof(word[2]));
	redraw(com.cvport, REMAP);
//	level_adjust(&(wfit[1]));
	return 0;
	}

int process_gauss2(int nb){
	int i;
/*	initseq(sourceseq);
	initseq(destseq);
	while ((source=seqnext(sourceseq))!=NULL){
		dest=seqnext(destseq);
		readfits(source,&gfit);
		process_gauss(nb);
		savefits(dest,&gfit);
		}
*/
	for (i=0;i<atoi(word[4]);++i){
		if (!readfits(buildfilename(word[3], i), &(gfit))){
				process_gauss(nb);
				savefits(buildfilename(word[4], i), &(gfit));
			}
		}
	return 0;
	}

int process_unsharp2(int nb){
	int i;

	for (i=0;i<=atoi(word[5]);++i){
		if(!readfits(buildfilename(word[3], i), &(gfit))){
				process_unsharp(nb);
				savefits(buildfilename(word[4], i), &(gfit));
			}
		}
	return 0;
	}

int process_fwhm(int nb){
	if (!com.drawn || com.drawing){
		if (nb==5){
			com.lx=atoi(word[1]);
			com.ly=atoi(word[2]);
			com.llx=atoi(word[3]);
			com.lly=atoi(word[4]);
			}
		else {
			sprintf(error_buffer,"fwhm: select a star or provide x,y,width,height\n");
			siril_error(error_buffer,0);
			return 1;
			}
		}

	fprintf(stderr,"%d %d %d %d \n",com.lx,com.ly,com.llx,com.lly);
	fwhm(&(gfit));
	redraw(com.cvport, REMAP);

	return 0;
	}

int process_crop(int nb){
	if (!com.drawn || com.drawing){
		if (nb==5){
			com.lx=atoi(word[1]);
			com.ly=atoi(word[2]);
			com.llx=atoi(word[3]);
			com.lly=atoi(word[4]);
			}
		else {
			sprintf(error_buffer,"Crop: select a region or provide x,y,width,height\n");
			siril_error(error_buffer,0);
			return 1;
			}
		}

	fprintf(stderr,"%d %d %d %d \n",com.lx,com.ly,com.llx,com.lly);
	crop(&(gfit));
	redraw(com.cvport, REMAP);
	
	return 0;
	}

int process_crop2(int nb){
	int i, err;
	char ofilename[256];

	if (!com.drawn){
		if (nb==8){
			com.lx=atoi(word[4]);
			com.ly=atoi(word[5]);
			com.llx=atoi(word[6]);
			com.lly=atoi(word[7]);
			}
		else {
			sprintf(error_buffer,"Crop2: select a region or provide x,y,width,height\n");
			siril_error(error_buffer,0);
			return 1;
			}
		}

	for (i=1;i<=atoi(word[3]);++i){	
		buildfilename(word[1],i);
		snprintf(ofilename,255,"%s%d",word[2],i);
		err=readfits(com.formname,&(gfit));
		if (err){
			siril_error("Crop2: aborted",0);
			return 1;
			}
		crop(&(gfit));
        savefits(ofilename,&(gfit));
		}
	redraw(com.cvport, REMAP);
	return 0;
	}

int process_medstack(int nb){
	stat_file(word[3],&(com.imagetype));
	medstack(word[1],atoi(word[2]),word[3]);
	strcpy(word[1],word[3]);
	process_load(1);
	return 0;
	}

int process_register(int nb){
	int cx, cy, debx, deby, size;
	int number;
	
	if (com.drawn){
		debx=com.lx;
		deby=com.cly-com.lly;
		size=max(com.llx,com.lly);
		cx=debx+com.llx/2;
		cy=deby+com.lly/2;
		}
	else{
		if (nb<6){
			siril_error("Select a rectangle or provide centerx centery size\n",0);
			return 0;
			}
		cx=atoi(word[3]);
		cy=atoi(word[4]);
		size=atoi(word[5]);
		}
	number=atoi(word[2]);
	isempty(word[1]);
	readseqfile(com.seq.name);
	_register(word[1],number,cx,cy,size,com.reglayer); 
	writeseqfile(com.seq.name);
	return 0;
	}

int process_cd(int nb){
	char buffer[256];

	strncpy(com.wd,word[1],255);
	changedir(com.wd);
	getcwd(buffer,255);
	siril_error(buffer,0);
	return 0;
	}
	
int process_wrecons(int nb){
	int i;
	float coef[7];
	char File_Name_Imag[250]="rawdata";
	char File_Name_Transform[256];

	snprintf(File_Name_Transform,255,"r_%s.wave",File_Name_Imag);
//	Nbr_Plan=atoi(word[1]);
	for (i=0;i<nb-1;++i){
		coef[i]=atof(word[i+1]);
		}

	wavelet_reconstruct_file (File_Name_Transform, coef,gfit.pdata[RLAYER]);
	if (isrgb(&gfit)){
		snprintf(File_Name_Transform,255,"g_%s.wave",File_Name_Imag);
		wavelet_reconstruct_file (File_Name_Transform, coef,gfit.pdata[GLAYER]);
		snprintf(File_Name_Transform,255,"b_%s.wave",File_Name_Imag);
		wavelet_reconstruct_file (File_Name_Transform, coef,gfit.pdata[BLAYER]);
		}
	redraw(com.cvport, REMAP);
	return 0;
	}	

int process_wavelet(int nb){
	char File_Name_Imag[250]="rawdata";
	char File_Name_Transform[256];
	int Type_Transform;
	int Nbr_Plan, maxplan, mins;
	float *Imag;

	Nbr_Plan = atoi(word[1]);
	Type_Transform = atoi(word[2]);

	mins = min (gfit.rx, gfit.ry);
	maxplan = log(mins) / log(2) - 2;

	if ( Nbr_Plan > maxplan ){
		snprintf(error_buffer,255,
			"Wavelet: maximum number of plans for this image size is %d\n", 
			maxplan);
		siril_error(error_buffer,0);
		return 1;
		}

	if(Type_Transform!=TO_PAVE_LINEAR && Type_Transform !=TO_PAVE_BSPLINE){
		snprintf(error_buffer,255,"Wavelet: type must be %d or %d\n",TO_PAVE_LINEAR,TO_PAVE_BSPLINE);
		siril_error(error_buffer,0);
		}

	Imag=(float *)malloc(gfit.rx*gfit.ry*sizeof(float));
	
	snprintf(File_Name_Transform,255,"r_%s.wave",File_Name_Imag);
	wavelet_transform_file (Imag, File_Name_Transform, 
					Type_Transform, Nbr_Plan, gfit.pdata[RLAYER]);
	if (isrgb(&gfit)){
		snprintf(File_Name_Transform,255,"g_%s.wave",File_Name_Imag);
		wavelet_transform_file (Imag, File_Name_Transform, 
						Type_Transform, Nbr_Plan, gfit.pdata[GLAYER]);
		
		snprintf(File_Name_Transform,255,"b_%s.wave",File_Name_Imag);
		wavelet_transform_file (Imag, File_Name_Transform, 
						Type_Transform, Nbr_Plan, gfit.pdata[BLAYER]);
		}
    free (Imag);

	return 0;
	}

int process_animate(int nb){
	int i, start, end, stride, err;
	double delay;

	start=atoi(word[2]);
	end=atoi(word[3]);
	delay=atof(word[4]);
	stride=max(1,atoi(word[5]));
	if(delay <0.02)
		delay=0.02;
	if(delay >1)
		delay=(double)1;
	for (i=start;i<=end && !(com.busy & ST_CANCEL);i+=stride){
		timing(0,"");
		buildfilename(word[1],i);
		err=readfits(com.formname,&(gfit));
		snprintf(com.status,255,"Displaying image %s %f", com.formname,delay);
		progress(0);
		if (err){
			siril_error("animate: error, ",0);
			}
		else{
			redraw(com.cvport,REMAP);
			}

		while (timing(1,"")<delay){
			progress(0);
			}
		}
	return 0;
	}

int process_log(int nb){
	loglut(&gfit);
	return 0;
	}

int process_thresh(int nb){
	int lo, hi;

	lo=atoi(word[1]);
	hi=atoi(word[2]);
	threshlo(&gfit,lo);
	threshhi(&gfit,hi);
	return 0;
	}

int process_threshlo(int nb){
	int lo;
	
	lo=atoi(word[1]);
	threshlo(&gfit,lo);
	return 0;
	}

int process_threshhi(int nb){
	int hi;

	hi=atoi(word[1]);
	threshhi(&gfit,hi);
	return 0;
	}

int process_nozero(int nb){
	int level;

	level=atoi(word[1]);
	
	nozero(&gfit, level);
	return 0;
	}

int process_ddp(int nb){
// combines an image division and a scalar multiplication.
	float coeff,sigma;
	unsigned level;

	level=atoi(word[1]);
	coeff=atof(word[2]);
	sigma=atof(word[3]);

	ddp(&gfit,level,coeff,sigma);
	redraw(com.cvport, REMAP);
	return 0;
	}

int xrgb(int nb, int layer){
	fprintf(stderr,"entering xrgb command, nb %d reading %s:%s:%s\n", nb, word[1], word[2], word[3]);
	switch(nb){
		case 2: // only one RGB filename is given
				// according to the value x of layer
				// perform xrgb composite
			readfits(word[1],wfit+4);
			copyfits(wfit+4,&gfit,CP_ALLOC|CP_EXPAND|CP_FORMAT,0); 	/* lrgb result goes to gfit */
			copyfits(wfit+4,wfit,CP_ALLOC|CP_EXTRACT,layer); 	/* l */
			copyfits(wfit+4,wfit+1,CP_ALLOC|CP_EXTRACT,RLAYER); /* r */
			copyfits(wfit+4,wfit+2,CP_ALLOC|CP_EXTRACT,GLAYER);	/* g */
			copyfits(wfit+4,wfit+3,CP_ALLOC|CP_EXTRACT,BLAYER);	/* b */
			break;

		case 4:
			readfits(word[1],wfit+1);	/* r */
			copyfits(wfit+1,&gfit,CP_ALLOC|CP_EXPAND|CP_FORMAT,0); 	/* lrgb result goes to gfit */
			readfits(word[2],wfit+2);	/* g */
			readfits(word[3],wfit+3);	/* b */
			copyfits(wfit+layer+1,wfit,CP_COPYA|CP_ALLOC|CP_FORMAT,layer); 	/* l */
			break;

		default:
			// error
			break;
		}
	fprintf(stderr,"invoking lrgb command from xrgb\n");
	lrgb(wfit, wfit+1, wfit+2, wfit+3, &gfit);
	redraw(com.cvport, REMAP);
	return 0;
	}

int process_rrgb(int nb){
	fprintf(stderr,"entering rrgb command\n");
	xrgb(nb,RLAYER);
	return 0;
	}

int process_grgb(int nb){
	fprintf(stderr,"entering grgb command\n");
	xrgb(nb,GLAYER);
	return 0;
	}

int process_brgb(int nb){
	fprintf(stderr,"entering brgb command\n");
	xrgb(nb,BLAYER);
	return 0;
	}
		
int process_lrgb(int nb){
//
// lrgb combines an rgb fits and a luminance fits
// through a short irruption in HSI space.
// takes 2 or 3 arguments. 
//			lrgb with 2 filenames l,rgb as arg1 and arg2
//			lrgb with 4 filenames l,r,g,b as arg[1234]
// 	
	fprintf(stderr,"entering lrgb command\n");
	switch(nb){
		case 2:
			readfits(word[2],wfit);
			readfits(word[3],wfit+4);
			copyfits(wfit+4,wfit+1,CP_EXTRACT,RLAYER); 	/* r */
			copyfits(wfit+4,wfit+2,CP_EXTRACT,GLAYER);	/* g */
			copyfits(wfit+4,wfit+3,CP_EXTRACT,BLAYER);	/* b */
			break;
			
		case 3:
			readfits(word[2],wfit);
			readfits(word[3],wfit+4);
			readfits(word[4],wfit+4);
			readfits(word[5],wfit+4);
			break;	

		default:
			// error
			break;
		}
	fprintf(stderr,"invoking lrgb function\n");
	copyfits(wfit,&gfit,CP_ALLOC|CP_EXPAND|CP_FORMAT,0); 	/* lrgb result goes to gfit */

	lrgb(wfit, wfit+1, wfit+2, wfit+3, &gfit);	
	redraw(com.cvport, REMAP);
	return 0;
}

int process_help(int nb){
	command *current=commande;
	
	while(current->process){
		snprintf (error_buffer,80,"%s",current->usage);
		siril_error(error_buffer,1);
		current++;
		}
	return 0;
	}


int process_exit(int nb){
	exit(0);
	}


int parsewords(int idx, char *ligne){
	int i;
	
	i=strcspn(ligne, " \n");
	if (i){
		snprintf(word[idx],++i,"%s",ligne);
		}
	return i;
	}


int processcommand(char *line){
	int i, j;
	int wordnb=0;

	com.busy|=ST_BUSY;
/*
	for (i=0; i<80 && line[i] && line[i]!=0x0a; ++i){
		if(! isalnum(line[i]) && !isblank(line[i]) && line[i]!='.' && line[i]!='/'){
			printf("                  *** Erreur : Caractre invalide : '%c' \n", line[i]);
			return 1;
			}
		}*/
	i=0;
	if (line[0]==0x0a)
		return 0;

	while (isblank(line[i])){
		if (line[i]==0x0a)
			return 0;
		++i;
		}	

	while((j=parsewords(wordnb, line+i))){
		i+=j;
		++wordnb;
		while (isblank(line[i])){
			++i;
			if (line[i]==0x0a)
				return 0;
			}
		}
/* Les mots de la commande sont rangs dans word ; 
 * il n'y a plus qu' les traiter ; surtout le premier
 */	
	j=MAXCOMMAND;
	while(strcmp (commande[--j].name,*word)){
		if (!j){
			snprintf(error_buffer,256,"*** Unknown command : '%s' \n", *word);
			siril_error(error_buffer,0);
			return 1 ;
			}
		}

	if(wordnb-1<commande[j].nbarg){
			snprintf(error_buffer,256,"   *** usage : %s\n", commande[j].usage);
			siril_error(error_buffer,0);
			return 1;
			}

	commande[j].process(wordnb);
	
	memset(word,0,MAXCHAR*MAXWORD);

	com.busy=ST_IDLE;
	return 0;
	}

